module Tut03 where

import Data.List (transpose)

{-T3.2-}
dimensions :: [[a]] -> (Int,Int)
dimensions [] = (0,0)
dimensions xs@(x:xt)
  | and [length x == length x' | x' <- xt] = (length xs, length x)
  | otherwise = (-1,-1)

isSquare :: [[a]] -> Bool
isSquare xs = let (m,n) = dimensions xs in
  m /= -1 && m == n

canAdd :: [[a]] -> [[a]] -> Bool
canAdd xs ys =
  let (m,n) = dimensions xs
      (m',n') = dimensions ys
  in m /= -1 && m == m' && n == n'

canMult :: [[a]] -> [[a]] -> Bool
canMult xs ys =
  let (_,n) = dimensions xs
      (m,_) = dimensions ys
  in m /= -1 && m == n 

diagonal :: [[a]] -> [a]
diagonal m = [m !! i !! i | i <- [0..length m - 1]]

cubeDiagonal :: [[[a]]] -> [a]
cubeDiagonal = diagonal . diagonal 

matrixAdd :: [[Integer]] -> [[Integer]] -> [[Integer]]
matrixAdd a b
 | canAdd a b = [[x+y | (x,y) <- zip aRow bRow] | (aRow,bRow) <- zip a b]
 | otherwise = []

matrixMult :: [[Integer]] -> [[Integer]] -> [[Integer]]
matrixMult a b
  | canMult a b = let bT = transpose b in
    [ [sum [x*y | (x,y) <- zip aRow bCol] | bCol <- bT ] | aRow <- a] 
  | otherwise = []

{-T3.3-}

splitList :: [a] -> ([a],[a])
splitList xs = splitAt n xs
  where
    n = length xs `div` 2

mergeLists :: ([Integer],[Integer]) -> [Integer]
mergeLists ([],bs) = bs
mergeLists (as,[]) = as
mergeLists (a:as,b:bs) = if a < b then a : mergeLists (as, b:bs)
                                  else b : mergeLists (a:as, bs)

mergeSort :: [Integer] -> [Integer]
mergeSort xs
  | length xs <= 1 = xs
  | otherwise = 
    let (as,bs) = splitList xs
        as' = mergeSort as
        bs' = mergeSort bs
    in mergeLists (as',bs')

adjacentPairs :: [a] -> [(a,a)]
adjacentPairs xs = zip xs (tail xs)

prop_mergeSort :: [Integer] -> Bool
prop_mergeSort xs = and [x <= y | (x,y) <- adjacentPairs (mergeSort xs)]

{-T4.4-}
collatz :: Integer -> [Integer]
collatz 1 = [1]
collatz n | n > 1 = n : collatz (if even n then n `div` 2 else 3 * n + 1)

prop_collatz :: Integer -> Bool
prop_collatz n = last (collatz n) == 1

prop_collatzOneHundred :: Bool
prop_collatzOneHundred = and [prop_collatz n | n<-[1..100]]
